/* Model Analysis by Posterior Predictive Simulations (MAPPS v1.0) 
	Jonathan Bollback Copyright 2002 */

/* MAPPS v1.01 */

/* Last modified 7/17/2002 */

/* 
Code written by:
	Jonathan P. Bollback
	Department of Biology
	University of Rochester
	Rochester, NY 14627 USA
	bollback@brahms.biology.rochester.edu
	bollback@biomail.ucsd.edu

Publication:
	Bollback, J. (2002) Bayesian Model Adequacy and Choice in Phylogenetics. Molecular Biology and Evolution. 19(7): 1171-1180.
	
Program Description:
	The program MAPPS was written to evaluate the fit of a group of phylogentic models to DNA sequence data (see below). Although not 
	implemented in this program the method could easily be extended to evaluate discrete or continuous phylogenetic models. The rationale
	behind this approach is that an adequate model should be able to predict future data (nucleotide site patterns). In the absence of
	future data the model's predictive ability is compared to the original data set. The model's predictive ability is evaluated through
	simulation under the model. Comparison of simulated (or predictive data sets) is evaluated using the multinomial test statistic. 
	This method is a Bayesian analogue of the parametric bootstrap and other frequentist approaches. For a more detailed description
	of the method see the publication listed above.
	
Models Implemented:
	1. JC69
	2. JC69 + gamma
	3. K2P
	4. K2P + gamma
	5. HKY85
	6. HKY85 + gamma
	7. GTR
	8. GTR + gamma
	
Liability Disclaimer/Source Code Distribution/Redistribution:
	The source code is provided as is with no waranty or gaurantee. The author (Jonathan Bollback) is not responsible for the performance
	of the code, quality or validity of the results produced, or any anomaolous effects it may have on the software users computer or
	information stored therein. This source code can be distributed as is as long as this comment section is included with it and it is
	for non-profit use. This code can not be sold or packaged on a media that is sold with out the permission of the author. This code
	can be freely modfied as long as the following conditions are met: 1. The code is clearly commented indicating that it is a modification
	of MAPPS v1.x by an author other than me, 2. The name of the modified software must clearly distinguish it from the unmodified version
	of MAPPS, 3. Modified versions can not be sold for profit, 4. This disclaimer must be included in the modified code or executable
	clearly accessible to software users, 5. Modified source code is freely available, and 6. publications resulting from modified code
	must cite the reference above. The source code distribution consists of the following files (the code in a number of the files
	was obtained from John P. Huelsenbeck, source of code is indicated in each file):
			1. mapps.c			(Bollback)
			2. linalg.c			(Huelsenbeck, EISPACK - see code)
			3. matrices.c		(Huelsenbeck)
			4. complex.c		(Huelsenbeck)
			5. gamma.c			(Yang 1994)
			6. matrices.h
			7. complex.h
			8. jph.h
			9. bollback.h
			10. gamma.h
			11. Makefile
			
Bugs:
	Please report any anomolous behavior or explicit BUGS found during executing the code or found in the code. By sending me an email
	with information regarding any BUGS I will be able to correct these mistakes and make a bug free version of the code available as
	soon as possible. I do not gaurantee that I will be able to make all fixes or that I will continue to support MAPPS in the future.
	If you have made a bug fix to the code please let me know what changes were made and what the problems was. Thank you.
	
*/


/* System Type */
#undef   WARRIOR_MAC		/* To compile a Mac Carbonized version using Metrowerks Codewarrior set this to #define */
						/* To compile for a UNIX system this statement must be #undef */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>

/* These are Codewarrior mac GUI specific, not for UNIX or other */
#if defined (WARRIOR_MAC)
	#include <console.h>
	#include <SIOUX.h>
	#include <MacTypes.h>
	#include <Fonts.h>
#endif

#include "linalg.h"
#include "matrices.h"
#include "complex.h"
#include "gamma.h"
#include "jph.h"
#include "bollback.h"		    /* Constant values required by program */


/* Debugging statements */
#undef  DEBUG
#undef  SHOW_TI_PROBS
#undef  DEBUG_GTR
#undef  SHOW_STRING
#undef  SITE_PATS
#undef  GAMMA_ONE				/* DO NOT IMPLEMENT THIS **** For each site and each node a new gamma distributed rate is chosen - hetergenous across the tree */
#define GAMMA_TWO				/* Each site is given a rate from the gamma distribution that does note change across the tree */

/* Special features */
#undef  BASE_TEST			    /* Multinomial test of base frequencies - as suggested by MBE reviewer #2 */

/* Define tree structure */
typedef struct node
	{
	struct node		      *left, *right, *anc;
	double		 length, treeSum, condLike[4];
	int	             label, index, flag, *seq;
	}
	TreeNode;

static TreeNode			   *node, *root, **upPassSeq, **downPassSeq;

/* Define tree structure */
typedef struct taxon_list
	{
	struct taxon_list		      *next, *back;
	int 		                    tree_label;
	char							label[100];
	}
	ListNode;

static ListNode			   *taxon_list, *start;

/* Global variables */
int    num = 1, nodeIndex = 0, reps=0,numOrigPatterns=0, numSimPatterns=0, S=0, T=0;
double sS = 0.0, sT = 0.0;
char   trees[MAXSTRING], inOriginal[100], inTreeFile[100], inParamFile[100], resultsName[100], fptrName[100], outName[100], debug[10];
char   numbers[11] = "0123456789", treeFile[MAXELEMS], paramFile[MAXELEMS];
unsigned char alpha_chars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ,./<>?;:'!@#$%^&*()-_=+";


void 	CalculateTransProb(double **q, double **ch_prob, int JC69, int HKY85, int K2P, int GTR, double v, double r, double kappa, double bFreq[4]);
int  	ChangeMatrix (double *eigenValues, double **eigvecs, double **inverseEigvecs, double t, double r, double **pt, int n);
int  	CompareChar(char c);
int  	ComplexChangeMatrix (double *eigenValues, double *eigvalsImag, complex **Ceigvecs, complex **CinverseEigvecs, int n, double **pt, double t, double r);
void 	DetermineBaseFrequencies(int nTaxa, int nSites, int **t, int **s, int original);
void 	DetermineSitePatterns(int nTaxa, int nSites, int **t, int **s, int original);
double 	**DoubleArrayMatrix(long nrl, long nrh, long ncl, long nch);
void 	FinishTree(TreeNode *p, int *i, double *splitTimes);
void 	Free_IntMatrix(int **d, long nrl, long ncl);
void 	Free_DoubleArraymatrix(double **d, long nrl, long ncl);
void 	GetDownPassSequence (TreeNode *p, int *i);
void    GetLabelsMatrix(int **a, int nTaxa, int nSites, char *s);
void    GetNumTaxaChars(int *nTaxa, int *nSites, char *string2);
void 	GetParameters(int *num_lines);
void 	GetTrees(int *num_lines);
void 	GetUpPassSequence(TreeNode *p, int *i);
int  	GetEigens (int n, double **q, double *eigenValues, double *eigvalsImag, double **eigvecs, double **inverseEigvecs, complex **Ceigvecs, complex **CinverseEigvecs);
void 	GTRChangeMat(double **q, double **ch_prob, double time, double r);
void    HelpMenu();
void 	HkyChangeMat(double **ch_prob, double time, double k, double r, double piA, double piC, double piG, double piT);
void 	InitializeConditionalLikelihoods(TreeNode *p, int nTaxa, int site);
int  	**IntMatrix(long nrl, long nrh, long ncl, long nch);
void 	JukesCantorChangeMat(double **ch_prob, double time, double r);
void 	Kimura2PChangeMat(double **ch_prob, double time, double k, double r);
void 	LabelTips(TreeNode *p);
void    ParseArray(char *array);
void    PrintError(int error);
void 	PrintHelp(int help);
void    ReadFile (FILE *inOR, char *string2);
int 	ReadParameterFile (double parameters[10], int JC69, int HKY85, int K2P, int GTR, int *bigGen, int *sampleFreq, int getBiggest, int sample, int initial, int usegamma);
void 	ReadTreeFile(TreeNode *p, int nTaxa, int gen);
void 	ReIndexNodes(TreeNode *p);
double 	Rnum(long int *seed);
void 	ShowTree(TreeNode *p, int n);
void 	SimulateData(TreeNode *p, long int *seed, int nTaxa, int numSites, double bFreq[4], double alpha, double kappa, double gtrRates[6], double m, int JC69, int HKY85, int K2P, int GTR, int usegamma);
void    SimulatePosterior(TreeNode *p, long int *seed, int JC69, int HKY85, int K2P, int GTR, int burnin, int usegamma, int draws);
void 	SetQMatrix (int n, double **a, double *pi, double gtrRates[6]);
void 	SetUpTree(TreeNode *p, int nTaxa);
void 	TestStatistic(int nT, int nS, int nTaxa, int nSites, int **T, int **S, int *bT, int *bS, int original);
int  	WhatChar(char c);
int  	WhichInt(char x);
void 	WhichNuc(int x);


int main (void)
{
	int            C1 = 0, C2 = 0, C3 = 0, C4 = 0, L, comp, model=0, usegamma=0, draws=1000, burnin=10000, JC69=0, HKY85=0, K2P=0, GTR=0, nTaxa, nSites, skip=0;
	long int	   seed;
	char 	       operation[1], Temp_seed[50], Temp_input[50], dummy[10];
	static char    s[] = "undefined";
	FILE		   *streamCheck;
	TreeNode *p;
	/*short familyID;*/

	/* Initializing time and date */
	struct tm *systime;
	time_t t, curTime;
	t = time(NULL);
	systime = localtime(&t);

#if defined (WARRIOR_MAC)
	/* Modifying SIOUX settings for this program */
	SIOUXSetTitle ("\pMAPPS v1.0");
	SIOUXSettings.autocloseonquit = 1;
	SIOUXSettings.asktosaveonclose = 0;
	SIOUXSettings.setupmenus = 0;
	/*SIOUXSettings.fontsize = 10;*/
	SIOUXSettings.fontface = bold;
	/*GetFNum("\pnewYork", &familyID);
	SIOUXSettings.fontid = familyID;*/
	SIOUXSettings.rows = 50;
	SIOUXSettings.columns = 83; 
#endif
	strcpy(inParamFile, s);
	strcpy(inTreeFile, s);
	strcpy(resultsName, s);
	strcpy(inOriginal, s);
	/* Initialize seed from current computer clock */
	curTime = time(NULL);
	seed  = (long int)curTime;
	if (seed < 0)
		seed = -seed;

	do
		{
		printf("\n**********************************************************************************\n");
		printf("*                                                                                *\n");
		printf("*       Model Analysis by Posterior Predictive Simulations (MAPPS v1.0)          *\n");
		printf("*                                    Version 1.0                                 *\n");
		printf("*                                                                                *\n");
		printf("*                                     Written by                                 *\n");
		printf("*                                Jonathan P. Bollback                            *\n");
		printf("*                                Department of Biology                           *\n");
		printf("*                               University of Rochester                          *\n");
		printf("*                                 Rochester, NY 14627                            *\n");
		printf("*                                                                                *\n");
		printf("**********************************************************************************\n");
		printf("\n");
		printf("                                %s",	asctime(systime));
		printf("\n");
		printf("RUN OPTIONS MENU:\n\n");
		printf("\tEnter the option to be modified <current values>.\n\n");
		printf("\tI\tInput data file name.                                    < %s >\n", inOriginal);
		printf("\tP\tPosterior distribution of model parameters file name.    < %s >\n", inParamFile);
		printf("\tT\tPosterior distribution of trees file name.               < %s >\n", inTreeFile);
		printf("\tO\tMAPPS output file name.                                  < %s >\n", resultsName);
		printf("\tM\tChoose substitution model.");
			if (model == 0)
				printf("                               < JC69 >\n");
			else if (model == 1)
				printf("                               < K2P >\n");
			else if (model == 2)
				printf("                               < HKY85 >\n");
			else if (model == 3)
				printf("                               < GTR >\n");
		printf("\tG\tUse gamma distributed rate correction.");
			if(usegamma==0){printf("                   < no >\n");}else{printf("                   < yes >\n");}
		printf("\tD\tNumber of draws to make from the posterior.              < %d >\n", draws);
		printf("\tB\tNumber of generations to be removed as burnin.           < %d >\n", burnin);
		
		printf("\tS\tInitial seed value.                                      < %ld >\n", seed);
		printf("\tR\tPerform model analysis.\n");
		printf("\tQ\tQuit program.\n");
		printf("\t?\tHELP MENU.\n\n\n\n\n\n");
		printf("> ");
		scanf("%s", &operation);

		if (operation[0] == 'I' || operation[0] == 'i')
			{
			printf ("  Enter file name containing the original data matrix...\n");
			printf ("> ");
			scanf ("%s", inOriginal);
			if ((streamCheck = fopen (inOriginal, "r+")) == NULL)     
				{
				strcpy(inOriginal, "undefined");
				printf("  ERROR: File not found in Application directory.\n");	
				printf("  Check spelling and verify that file is located in the correct directory.\n");	
				printf("  Type C and press RETURN to continue: ");	
				scanf("%s", &dummy);
				}
			fclose(streamCheck);
			}
		else if (operation[0] == 'P' || operation[0] == 'p')
			{
			printf ("  Enter file name containing the posterior distribution of parameters...\n");
			printf ("> ");
			scanf ("%s", inParamFile);
			if ((streamCheck = fopen (inParamFile, "r+")) == NULL)     
				{
				strcpy(inParamFile, "undefined");
				printf("  ERROR: File not found in Application directory.\n");	
				printf("  Check spelling and verify that file is located in the correct directory.\n");	
				printf("  Type C and press RETURN to continue: ");	
				scanf("%s", &dummy);
				}
			fclose(streamCheck);
			}
		else if (operation[0] == 'T' || operation[0] == 't')
			{
			printf ("  Enter file name containing the posterior distribution of trees...\n");
			printf ("> ");
			scanf ("%s", inTreeFile);
			if ((streamCheck = fopen (inTreeFile, "r+")) == NULL)     
				{
				strcpy(inTreeFile, "undefined");
				printf("  ERROR: File not found in Application directory.\n");	
				printf("  Check spelling and verify that file is located in the correct directory.\n");	
				printf("  Type C and press RETURN to continue: ");	
				scanf("%s", &dummy);
				}
			fclose(streamCheck);
			}
		else if (operation[0] == 'O' || operation[0] == 'o')
			{
			printf ("  Enter file name for program output...\n");
			printf ("> ");
			scanf ("%s", resultsName);
			}
		else if (operation[0] == 'M' || operation[0] == 'm')
			{
			printf ("  Choose desired substitution model...\n");
			printf ("    Option: Model\n");
			printf ("       0  : JC69\n");
			printf ("       1  : K2P\n");
			printf ("       2  : HKY85\n");
			printf ("       3  : GTR\n");
			printf ("> ");
			scanf ("%s", &Temp_input);
			model = atoi(Temp_input);
			if (model < 0 || model > 3)
				{
				PrintError(4);
				model = 0;
				}
			}
		else if (operation[0] == 'G' || operation[0] == 'g')
			{
			printf ("  Use gamma distributed rate correction (y/n)?\n");			
			printf ("> ");
			scanf ("%s", &Temp_input);
			if (Temp_input[0] == 'y' || Temp_input[0] == 'Y' || Temp_input[0] == 'n' || Temp_input[0] == 'N')
				{
				if (Temp_input[0] == 'y' || Temp_input[0] == 'Y' )
					usegamma = 1;
				else if (Temp_input[0] == 'n' || Temp_input[0] == 'N' )
					usegamma = 0;
				}
			else
				{
				PrintError(5);
				usegamma = 0;
				}
			}
		else if (operation[0] == 'D' || operation[0] == 'd')
			{
			printf ("  Enter number of draws to make from the posterior...\n");
			printf ("> ");
			scanf ("%s", &Temp_input);
			draws = atoi(Temp_input);
			if (draws == 0)
				{
				PrintError(6);
				draws = 100;
				}
			else if (draws < 0)
				{
				PrintError(7);
				draws = 100;
				}
			}
		else if (operation[0] == 'B' || operation[0] == 'b')
			{
			printf ("  Enter number of generations to exclude as burnin...\n");
			printf ("> ");
			scanf ("%s", &Temp_input);
			burnin = atoi(Temp_input);
			if (burnin == 0)	 					/* WARNING: Warn user that method is not valid when samples are drawn from a chain not at stationarity. */
				PrintError(8);
			else if (burnin < 0) 					/* ERROR: This variable cannot take on a negative value. */
				{
				PrintError(9);
				burnin = 1000;
				}
			/* Error check that burnin is not greater than number of posterior samples MUST BE DONE ONCE POSTERIOR FILE IS READ*/
			}
		else if (operation[0] == 'S' || operation[0] == 's')
			{
			printf ("  Enter initial seed value...\n");
			printf ("> ");
			scanf ("%s", &Temp_seed);
			L = strlen(Temp_seed);
			comp = memcmp(Temp_seed, alpha_chars, L);
			if (comp > 0)
				{
				PrintError(0);
				}
			else if (comp < 0)
				{
				seed = atol(Temp_seed);
				}
			if (seed < 0)
				{
				PrintError(10);
				/* Initialize seed from current computer clock */
				curTime = time(NULL);
				seed  = (long int)curTime;
				if (seed < 0)
					seed = -seed;
				}
			}
		else if (operation[0] == 'R' || operation[0] == 'r')
			{
			/* Run program code goes here - do some checking before proceeding */
			C1 = strcmp(inParamFile, "undefined");
			C2 = strcmp(resultsName, "undefined");
			C3 = strcmp(inTreeFile, "undefined");
			C4 = strcmp(inOriginal, "undefined");
			if (C1 == 0)
				{
				PrintError(1);
				skip = 1;				
				}
			if (C2 == 0)
				{
				PrintError(11);				
				skip = 1;				
				}
			if (C3 == 0)
				{
				PrintError(2);				
				skip = 1;				
				}
			if (C4 == 0)
				{
				PrintError(12);
				skip = 1;				
				}
			/* Establish model */
			if (model == 0)
				{
				JC69 = 1;
				HKY85 = 0;
				K2P = 0;
				GTR = 0;
				}
			else if (model == 1)
				{
				JC69 = 0;
				HKY85 = 0;
				K2P = 1;
				GTR = 0;
				}
			else if (model == 2)
				{
				JC69 = 0;
				HKY85 = 1;
				K2P = 0;
				GTR = 0;
				}
			else if (model == 3)
				{
				JC69 = 0;
				HKY85 = 0;
				K2P = 0;
				GTR = 1;
				}				
			if (skip == 0)
				{
				#if defined (WARRIOR_MAC)
					clrscr();
				#endif
				/* Pass collected information about files and run conditions to function SimulatePosterior */
				SimulatePosterior(p, &seed, JC69, HKY85, K2P, GTR, burnin, usegamma, draws);
				nSites = S;
				nTaxa = T;
				printf("\n\nType C and press RETURN to proceed to the MAIN MENU:\n");	
				printf("> ");
				scanf("%s", &dummy);
				}
			}			
		else if (operation[0] == '?')
			{
			HelpMenu();
			}
		else if (operation[0] == 'Q' || operation[0] == 'q')
			{
			printf("\nQuiting program....\n");
			break;
			}
		else
			{
			PrintError(3);
			}
		#if defined (WARRIOR_MAC)
		clrscr();
		#endif
		}while(operation[0] != 'Q' || operation[0] != 'q');

	return 0;
}


void CalculateTransProb(double **q, double **ch_prob, int JC69, int HKY85, int K2P, int GTR, double v, double r, double kappa, double bFreq[4])
{
	if (JC69 == YES)
		{
		JukesCantorChangeMat(ch_prob, v, r);
		}
	else if (HKY85 == YES)
		{
		HkyChangeMat(ch_prob, v, kappa, r, bFreq[0], bFreq[1], bFreq[2], bFreq[3]);
		}
	else if (K2P == YES)
		{
		Kimura2PChangeMat(ch_prob, v, kappa, r);
		}
	else if (GTR == YES)
		{
		GTRChangeMat(q, ch_prob, v, r);
		}
}


int ChangeMatrix (double *eigenValues, double **eigvecs, double **inverseEigvecs, double t, double r, double **pt, int n)
{
	int			i, j, k, rc;
	double		sum;

	rc = NO_ERROR;
	for(i=0; i<n; i++)
		{
		for(j=0; j<n; j++)
			{
			sum = 0.0;
			for(k=0; k<n; k++)
				{
				sum += (eigvecs[i][k] * inverseEigvecs[k][j] * exp(eigenValues[k] * t * r));
				if (sum < 0.0)
					pt[i][j] = 0.0;
				else
					pt[i][j] = sum;
				}
			}
		}
   	return (rc);
}


int CompareChar(char c)
{
	int i;

	for (i = 0; i < 10; i++)
		{
		if (c == numbers[i])
			return (1);
		}
	return(0);
}


int ComplexChangeMatrix (double *eigenValues, double *eigvalsImag, complex **Ceigvecs, complex **CinverseEigvecs, int n, double **pt, double t, double r)
{
	int			i, j, k, rc;
	complex		 **Cwork;

	rc = NO_ERROR;
	Cwork = pscmatrix(n);
	for(i=0; i<n; i++)
		{
		for(j=0; j<n; j++)
			{
			Cwork[i][j].re = 0.0;
			Cwork[i][j].im = 0.0;
			for(k = 0; k < n; k++)
				{
				Cwork[i][j] = Cadd(Cwork[i][j], Cmul(Ceigvecs[i][k], Cmul(CinverseEigvecs[k][j], Cexp(RCmul(t*r, Complex(eigenValues[k], eigvalsImag[k]))))));
				}
			}
		}
	for(i=0; i<n; i++)
		{
		for(j=0; j<n; j++)
			{
			pt[i][j] = Cwork[i][j].re;
			if(fabs(Cwork[i][j].im) > 0.000000000001)
				{
	    		rc = ERROR;
				}
			}
		}
	free_pscmatrix (Cwork);
	return (rc);
}


double **DoubleArrayMatrix(long nrl, long nrh, long ncl, long nch)
{
	long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
	double **d;

	/* allocate pointers to rows */
	d = (double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	if (!d)
		{
		printf("Could not allocate pointer to rows for dmatrix!\n");
		exit(1);
		}
	d += NR_END;
	d -= nrl;
	
	/* allocate rows and set pointers to them */
	d[nrl] = (double *)malloc((size_t)(((nrow*ncol)+NR_END)*sizeof(double)));
	if (!d[nrl])
		{
		printf("Could not allocate rows and set pointers to them!\n");
		exit(1);
		}
	d[nrl] += NR_END;
	d[nrl] -= ncl;
	
	for(i = nrl + 1; i <= nrh; i++)
		{
		d[i] = d[i-1] + ncol;
		}
	
	return (d);
}


void DetermineBaseFrequencies(int nTaxa, int nSites, int **t, int **s, int original)
{
	int i, j, origBaseFreqs[4], simBaseFreqs[4], **origPatterns, **simPatterns;

	for (i = 0; i < 4; i++)
		origBaseFreqs[i] = simBaseFreqs[i] = 0;

	for (i = 0; i < nTaxa; i++)
		{
		for (j = 0; j < nSites; j++)
			{
			if (original == YES)
				{
				if (t[i][j] == 0)
					origBaseFreqs[0]++;
				else if (t[i][j] == 1)
					origBaseFreqs[1]++;
				else if (t[i][j] == 2)
					origBaseFreqs[2]++;
				else if (t[i][j] == 3)
					origBaseFreqs[3]++;
				}
			else if (original == NO)
				{
				if (s[i][j] == 0)
					simBaseFreqs[0]++;
				else if (s[i][j] == 1)
					simBaseFreqs[1]++;
				else if (s[i][j] == 2)
					simBaseFreqs[2]++;
				else if (s[i][j] == 3)
					simBaseFreqs[3]++;
				}
			}
		}
	TestStatistic(numOrigPatterns, numSimPatterns, nTaxa, nSites, origPatterns, simPatterns, origBaseFreqs, simBaseFreqs, original);
}


void DetermineSitePatterns(int nTaxa, int nSites, int **t, int **s, int original)
{
	int i, j, k, l, same[MAXTAXA], b1[4], b2[4], match = YES;
	int **origPatterns, **simPatterns;


	if (original == YES)
		{
		origPatterns = IntMatrix(0, (long)(nTaxa+1), 0, (long)nSites);
		numOrigPatterns = 0;
		}
	else if (original == NO)
		{
		simPatterns  = IntMatrix(0, (long)(nTaxa+1), 0, (long)nSites);
		numSimPatterns  = 0;
		}

	for (i = 0; i < nTaxa+1; i++)
		{
		for (j = 0; j < nSites; j++)
			{
			if (original == YES)
				origPatterns[i][j] = 0;
			else if (original == NO)
				simPatterns[i][j]  = 0;
			}
		}

	/* Determine number of patterns and number of sites exhibiting this pattern for original data matrix and simulated matrix */
	for (i = 0; i < nSites; i++)
		{
		if (original == YES)
			{
			if (numOrigPatterns == 0)
				{
				for (l = 0; l < nTaxa; l++)
					{
					origPatterns[l][0] = t[l][i];
					}
				origPatterns[nTaxa+1][0] = 1;
				numOrigPatterns++;
				}
			else if (numOrigPatterns > 0)
				{
				for (k = 0; k < numOrigPatterns; k++)
					{
					match = YES;
					for (l = 0; l < nTaxa; l++)
						same[l] = NO;
					for (l = 0; l < nTaxa; l++)
						{
						if (origPatterns[l][k] == t[l][i])
							same[l] = YES;
						else
							same[l] = NO;
						}
					for (l = 0; l < nTaxa; l++)
						{
						if (same[l] == NO)
							match = NO;
						}
					if (match == YES)
						{
						origPatterns[nTaxa+1][k]++;
						break;
						}
					else if (k == (numOrigPatterns-1))
						{
						for (l = 0; l < nTaxa; l++)
							{
							origPatterns[l][numOrigPatterns] = t[l][i];
							}
						origPatterns[nTaxa+1][numOrigPatterns] = 1;
						numOrigPatterns++;
						break;
						}
					}
				}
			}
		else if (original == NO)
			{
			if (numSimPatterns == 0)
				{
				for (l = 0; l < nTaxa; l++)
					{
					simPatterns[l][0] = s[l][i];
					}
				simPatterns[nTaxa+1][0] = 1;
				numSimPatterns++;
				}
			else if (numSimPatterns > 0)
				{
				for (k = 0; k < numSimPatterns; k++)
					{
					match = YES;
					for (l = 0; l < nTaxa; l++)
						same[l] = NO;
					for (l = 0; l < nTaxa; l++)
						{
						if (simPatterns[l][k] == s[l][i])
							same[l] = YES;
						else
							same[l] = NO;
						}
					for (l = 0; l < nTaxa; l++)
						{
						if (same[l] == NO)
							match = NO;
						}
					if (match == YES)
					    {
						simPatterns[nTaxa+1][k]++;
						break;
						}
					else if (k == (numSimPatterns-1))
						{
						for (l = 0; l < nTaxa; l++)
							{
							simPatterns[l][numSimPatterns] = s[l][i];
							}
						simPatterns[nTaxa+1][numSimPatterns] = 1;
						numSimPatterns++;
						break;
						}
					}
				}
			}
		}

#	if defined (SITE_PATS)
	for (i = 0; i < numOrigPatterns; i++)
		{
		for (k = 0; k < nTaxa; k++)
			{
			printf("%d ", origPatterns[k][i]);
			}
		printf(" => %d\n", origPatterns[k+1][i]);
		}
#	endif
	
	TestStatistic(numOrigPatterns, numSimPatterns, nTaxa, nSites, origPatterns, simPatterns, b1, b2, original);

	if (original == YES)	
		Free_IntMatrix(origPatterns, 0, 0);
	else if (original == NO)
		Free_IntMatrix(simPatterns, 0, 0);
}


void FinishTree(TreeNode *p, int *i, double *splitTimes)
{
	if (p != NULL)
		{
		if (p->left == NULL && p->right == NULL && p->anc != NULL)
			{
			p->length  = 1.0 - (p->anc->treeSum);
			p->treeSum = 1.0;
			}
		else if (p->left != NULL && p->right != NULL && p->anc != NULL)
			{
			p->length  = splitTimes[(*i)] - (p->anc->treeSum);
			p->treeSum = splitTimes[(*i)++];
			}
		else if (p == root)
			p->length = splitTimes[(*i)++];
		FinishTree(p->left, i, splitTimes);
		FinishTree(p->right, i, splitTimes);
		}
}


void Free_IntMatrix(int **d, long nrl, long ncl)
{
	free((FREE_ARG)(d[nrl]+ncl-NR_END));
	free((FREE_ARG)(d+nrl-NR_END));
}



void Free_DoubleArraymatrix(double **d, long nrl, long ncl)
{
	free((FREE_ARG)(d[nrl]+ncl-NR_END));
	free((FREE_ARG)(d+nrl-NR_END));
}


void GetDownPassSequence(TreeNode *p, int *i)
{
	if (p != NULL)
		{
		GetDownPassSequence(p->left,  i);
		GetDownPassSequence(p->right, i);
		downPassSeq[(*i)++] = p;
		}
}


void GetLabelsMatrix(int **a, int nTaxa, int nSites, char *s)
{
	int i, j, k, incrementor=0, numPipes=0, evaluate;								/* MAX number of pipes encountered for n taxa is (2*n+1), i.e., the number of tokens is (2*n) */
	char                     *tempString, c;
	ListNode *p;
	
	tempString = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!tempString)
		{
		printf("ERROR: Problem allocating string2 in function 'GetLabelsMatrix'.\n");
		exit(1);
		}
		
	/* Initialize the list to NULL for pointers */
	for (i = 0; i < nTaxa; i++)
		{
		p = taxon_list + i;
		p->next = NULL;
		p->back = NULL;
		}	

	for (i = 0; i < MAXELEMS; i++)
		{
		j = 0;
		do
			{
			tempString[j] = s[i];
			j++;
			i++;
			} while ((s[i] != PIPE) || (i == MAXELEMS-1));
		tempString[j] = NULLTERM;
		if (i == MAXELEMS-1)
			{
			printf("ERROR: Attempting to exceed MAXELEMS in function 'GetLabelsMatrix'.\n");
			exit(1);
			}
		else if ( (evaluate = strcmp(tempString, "matrix")) == 0)		/* Next token should be taxon label for sequence ONE if entering for the first time */
			{
			i++;
			do
				{
				j = 0;
				do
					{
					tempString[j] = s[i];
					j++;
					i++;
					} while ((s[i] != PIPE) || (i == MAXELEMS-1));
				tempString[j] = NULLTERM;
				if (numPipes % 2 == 0)								/* Get taxon labels */
					{
					p = taxon_list + incrementor;
					p->next = taxon_list + (incrementor + 1);
					if (incrementor != 0)
						{
						p->back = taxon_list + (incrementor - 1);
						}
					strcpy(p->label, tempString);
					}
				else if (numPipes % 2 != 0)							/* Get matrix data */
					{
					for (k = 0; k < nSites; k++)
						{
						c = tempString[k];
						a[incrementor][k] = WhichInt(c);
						}
					incrementor++;
					}
				i++;
				numPipes++;
				}while(numPipes < (2*nTaxa)+1);						/* When last pipe is encountered exit */
			break;
			}
		}

	free(tempString);
}


void GetNumTaxaChars(int *nTaxa, int *nSites, char *string2)
{
	int  i, j, offSet = 0, evaluate;
	char *string, *tempString;

	string = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!string)
		{
		printf("ERROR: Problem allocating string in function 'ReadOriginalData'.\n");
		exit(1);
		}
	tempString = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!tempString)
		{
		printf("ERROR: Problem allocating string in function 'ReadOriginalData'.\n");
		exit(1);
		}
	tempString[0] = NULLTERM;

	/* Find number of taxa, number of characters, taxon labels, and data matrix */
	offSet = strlen(string2);
	for (i = 0; i < offSet; i++)
		{
		if (string2[i] != '|')
			{
			j = 0;
			while (string2[i] != '|')		/* Transfer token to temporary storage */
				{
				tempString[j] = string2[i];
				j++;
				i++;
				}
			tempString[j] = NULLTERM;
			if ( (evaluate = strcmp(tempString, "ntax")) == 0)
				{
				i++;
				j = 0;
				while (string2[i] != '|')		/* Transfer token to temporary storage */
					{
					tempString[j] = string2[i];
					j++;
					i++;
					}
				tempString[j] = NULLTERM;
				sscanf(tempString, "%d", nTaxa);
				}
			else if ( (evaluate = strcmp(tempString, "nchar")) == 0)
				{
				i++;
				j = 0;
				while (string2[i] != '|')		/* Transfer token to temporary storage */
					{
					tempString[j] = string2[i];
					j++;
					i++;
					}
				tempString[j] = NULLTERM;
				sscanf(tempString, "%d", nSites);
				}
			}
		else if (string2[i] == NULLTERM || i == offSet-1)
			{
			break;
			}
		}
	free(string);
	free(tempString);
}


void GetParameters(int *num_lines)
{
	int     i, n;
	char       x;
	FILE 	 *in;
	
	/* Open file stream and pointer to file */
	if ((in = fopen (inParamFile, "r+")) == NULL)     
		{
		printf ("ERROR: Could not open parameter file.\n");
		exit (-1);
		}
	
	/* Parse file into a string where lines are seperated by '|' in the array */
	i = n = 0;
	while ((x = fgetc(in)) != EOF)
		{
		if (x == '\n')					/* Macintosh end of line character */
			{
			paramFile[i] = '|';
			n++;
			}
		else if (x == '\r')				/* UNIX end of line character */
			{
			paramFile[i] = '|';
			n++;
			}
		else
			{
			paramFile[i] = x;
			}
		i++;
		if (i == MAXELEMS)			/* Check that the program isn't reading values from the file into illegal blocks of memory */
			{
			printf("Not enough storage available in parameter file array. Reset precompiler variable MAXELEMS to a larger value and recompile. Current value is %d.\n", MAXELEMS);
			exit(1);
			}
		}
	paramFile[i] = NULLTERM;				/* NULL terminate the string */
	*num_lines  =    n;					/* Return number of lines in array from file */

	fclose(in);
}


void GetTrees(int *num_lines)
{
	int     i, n;
	char       x;
	FILE   *inTF;
	
	/* Open file stream and pointer to file */
	if ((inTF = fopen (inTreeFile, "r")) == NULL)     
		{
		printf ("ERROR: Could not open trees file.\n");
		exit (-1);
		}
	
	/* Parse file into a string where lines are seperated by '|' in the array */
	i = n = 0;
	while ((x = fgetc(inTF)) != EOF)
		{
		if (x == '\n')					/* Macintosh end of line character */
			{
			treeFile[i] = '|';
			n++;
			}
		else if (x == '\r')				/* UNIX end of line character */
			{
			treeFile[i] = '|';
			n++;
			}
		else
			{
			treeFile[i] = x;
			}
		i++;
		if (i == MAXELEMS)			/* Check that the program isn't reading values from the file into illegal blocks of memory */
			{
			printf("Not enough storage available in parameter file array. Reset precompiler variable MAXELEMS to a larger value and recompile. Current value is %d.\n", MAXELEMS);
			exit(1);
			}
		}
	treeFile[i] = NULLTERM;				/* NULL terminate the string */
	*num_lines  =    n;					/* Return number of lines in array from file */

	fclose(inTF);
}


void GetUpPassSequence(TreeNode *p, int *i)
{
	if (p != NULL)
		{
		upPassSeq[(*i)++] = p;
		GetUpPassSequence(p->left, i);
		GetUpPassSequence(p->right, i);
		}	
}


int GetEigens (int n, double **q, double *eigenValues, double *eigvalsImag, double **eigvecs, double **inverseEigvecs, complex **Ceigvecs, complex **CinverseEigvecs)
{
	int			i, j, rc, *iwork, isComplex;
	double		**mwork, *dwork;
	complex 	**Cwork, *Ccol;

	/* allocate memory */
	dwork           = (double *)malloc((size_t) (n * sizeof(double)));
	iwork           = (int *)malloc((size_t) (n * sizeof(int)));

	/* calculate eigenvalues and eigenvectors */
	isComplex = NO;
	rc = EigenRealGeneral (n, q, eigenValues, eigvalsImag, eigvecs, iwork, dwork);
	if (rc != NO_ERROR)
		{
		if (rc == RC_COMPLEX_EVAL)
			{
			isComplex = YES;
			}
		}

	if (isComplex == NO)
		{
		mwork = psdmatrix (n);
		copy_psdmatrix (eigvecs, mwork, n);
		InvertMatrix (mwork, n, dwork, iwork, inverseEigvecs);
		free_psdmatrix (mwork);
		}
	else
		{
		for(i=0; i<n; i++)
			{
			if (eigvalsImag[i] == 0.0)
				{ 
				for(j=0; j<n; j++)
					{
					Ceigvecs[j][i].re = eigvecs[j][i];
					Ceigvecs[j][i].im = 0.0;
					}
				}
			else if (eigvalsImag[i] > 0.0)
				{ 
				for (j=0; j<n; j++)
					{
					Ceigvecs[j][i].re = eigvecs[j][i];
					Ceigvecs[j][i].im = eigvecs[j][i + 1];
					}
				}
			else if (eigvalsImag[i] < 0.0)
				{ 
				for (j=0; j<n; j++)
					{
					Ceigvecs[j][i].re =  eigvecs[j][i-1];
					Ceigvecs[j][i].im = -eigvecs[j][i];
					}
				}
			}
		Ccol = (complex *)malloc((size_t) (n * sizeof(complex)));
		Cwork = pscmatrix (n);
		copy_pscmatrix (Ceigvecs, Cwork, n);
		ComplexInvertMatrix (Cwork, n, dwork, iwork, CinverseEigvecs, Ccol);
		free (Ccol);
		free_pscmatrix (Cwork);
		}

	free (dwork);
	free (iwork);
	return (isComplex);
}


void GTRChangeMat(double **q, double **ch_prob, double time, double r)
{
	int 		n=4, isComplex = NO;
	double		*eigenValues, *eigvalsImag, **eigvecs, **inverseEigvecs, **f, **s;
	complex 	**Ceigvecs, **CinverseEigvecs;

#	if defined (DEBUG_GTR)
	int 		i, j; 
#	endif

	f               = psdmatrix (n);
	s               = psdmatrix (n);
	eigenValues     = (double *)malloc((size_t) (n * sizeof(double)));
	eigvalsImag     = (double *)malloc((size_t) (n * sizeof(double)));
	eigvecs         = psdmatrix (n);
	inverseEigvecs  = psdmatrix (n);
	Ceigvecs        = pscmatrix (n);
	CinverseEigvecs = pscmatrix (n);
	
	/* Get eigen values */
	isComplex = GetEigens (n, q, eigenValues, eigvalsImag, eigvecs, inverseEigvecs, Ceigvecs, CinverseEigvecs);
	
	/* Calculate transition probabilities */
	if (isComplex == NO)
		{
		if (ChangeMatrix (eigenValues, eigvecs, inverseEigvecs, time*r, 1.0, ch_prob, n) == ERROR)
			printf ("Problem calculating transition probabilities.\n");
		}
	else
		{
		if (ComplexChangeMatrix (eigenValues, eigvalsImag, Ceigvecs, CinverseEigvecs, n, ch_prob, time*r, 1.0) == ERROR)
			printf ("Problem calculating transition probabilities for complex eigen values.\n");
		}

#	if defined (DEBUG_GTR)
	for (i = 0; i < STATES; i++)
		{
		for (j = 0; j < STATES; j++)
			{
			printf("%lf ", ch_prob[i][j]);
			}
		printf("\n");
		}
#	endif

	free_psdmatrix (f);
	free_psdmatrix (s);
	free (eigenValues);
	free (eigvalsImag);
	free_psdmatrix (eigvecs);
	free_psdmatrix (inverseEigvecs);
	free_pscmatrix (Ceigvecs);
	free_pscmatrix (CinverseEigvecs);
}


void HelpMenu()
{
	char help[1];

	do
		{
		#if defined (WARRIOR_MAC)
			clrscr();
		#else
			printf("\n\n");
		#endif
		printf("\n**********************************************************************************\n");
		printf("*                                                                                *\n");
		printf("*       Model Analysis by Posterior Predictive Simulations (MAPPS v1.0)          *\n");
		printf("*                                    Version 1.0                                 *\n");
		printf("*                                                                                *\n");
		printf("*                                     Written by                                 *\n");
		printf("*                                Jonathan P. Bollback                            *\n");
		printf("*                                Department of Biology                           *\n");
		printf("*                               University of Rochester                          *\n");
		printf("*                                 Rochester, NY 14627                            *\n");
		printf("*                                                                                *\n");
		printf("**********************************************************************************\n");
		printf("\n");
		printf("\n");
		printf("HELP MENU:\n\n");
		printf("\tEnter topic.\n\n");
		printf("\tV\tOverview of method.\n");
		printf("\tI\tInput files.\n");
		printf("\tO\tOutput file.\n");
		printf("\tA\tAuthor contact information.\n");
		printf("\tK\tAcknowledgements.\n");
		printf("\tL\tLiability disclaimer and software distribution.\n");
		printf("\tM\tReturn to RUN OPTIONS MENU.\n\n\n\n\n\n\n\n\n\n\n");
		printf("> ");
		scanf("%s", &help);
		if (help[0] == 'I' || help[0] == 'i') /* Input files */
			{
			PrintHelp(0);
			}
		else if (help[0] == 'O' || help[0] == 'o') /* Output file */
			{
			PrintHelp(1);
			}
		else if (help[0] == 'A' || help[0] == 'a') /* Author contact information */
			{
			PrintHelp(2);
			}
		else if (help[0] == 'K' || help[0] == 'k') /* Acknowledgements */
			{
			PrintHelp(4);
			}
		else if (help[0] == 'L' || help[0] == 'l') /* Liability and softwaredistribution */
			{
			PrintHelp(3);
			}
		else if (help[0] == 'H' || help[0] == 'h') /* Help Andrea */
			{
			PrintHelp(5);
			}
		else if (help[0] == 'V' || help[0] == 'v') /* Overview of method */
			{
			PrintHelp(6);
			}
		else if (help[0] == 'M' || help[0] == 'm') /* Return to run options menu  */
			{
			break;
			}
		}while(help[0] != 'M' || help[0] != 'm');
}


void HkyChangeMat (double **ch_prob, double time, double k, double r, double piA, double piC, double piG, double piT)
{
	int		i, j;
	double	t, u, w, x, y, z, beta, bigPi_j[4], pij, bigPij, pis[4];

	/* rescale beta */
	pis[0] = piA;
	pis[1] = piC;
	pis[2] = piG;
	pis[3] = piT;
	beta = 0.5 / ((piA + piG)*(piC + piT) + k*((piA*piG) + (piC*piT)));

	bigPi_j[0] = piA + piG;
	bigPi_j[1] = piC + piT;
	bigPi_j[2] = piA + piG;
	bigPi_j[3] = piC + piT;
	t = time * r;

	for (i=0; i<4; i++)
		{
		for (j=0; j<4; j++)
			{
			bigPij = bigPi_j[j];
			pij = pis[j];
			u = 1.0/bigPij - 1.0;
			w = -beta * (1.0 + bigPij * (k - 1.0));
			x = exp(-beta * t);
			y = exp(w * t);
			z = (bigPij - pij) / bigPij;
			if (i == j)
				ch_prob[i][j] = pij + pij * u * x + z * y;
			else if ((i == 0 && j == 2) || (i == 2 && j == 0) || (i == 1 && j == 3) || (i == 3 && j == 1))
				ch_prob[i][j] = pij + pij * u * x - (pij/bigPij) * y;
			else
				ch_prob[i][j] = pij * (1.0 - x);
			}
		}
		
#	if defined (SHOW_TI_PROBS)
	printf ("Change probabilities (HKY 85, time = %lf):\n", time);
	for (i=0; i<4; i++)
		{
		for (j=0; j<4; j++)
			{
			printf ("%lf ", ch_prob[i][j]);
			}
		printf ("\n");
		}
	exit(1);
#	endif
}


void InitializeConditionalLikelihoods(TreeNode *p, int nTaxa, int site)
{
	int i, j;
	
	for (i = 0; i < ((2*nTaxa)-1); i++)
		{
		p = upPassSeq[i];
		if (p->left == NULL && p->right == NULL)	/* At tip */
			{
			for (j = 0; j < STATES; j++)
				{
				if (p->seq[site] == j)
					p->condLike[j] = 1.0;
				else
					p->condLike[j] = 0.0;
				}
			}
		else
			{
			for (j = 0; j < STATES; j++)
				p->condLike[j] = 0.0;
			}
		}
}


void JukesCantorChangeMat(double **ch_prob, double time, double r)
{
	int i, j;

	for (i = 0; i < 4; i++)
		{
		for (j = 0; j < 4; j++)
			{
			if (i == j)
				{
				ch_prob[i][j] = 0.25 + (0.75*exp(-(4.0/3.0)*time*r));
				}
			else if (i != j)
				{
				ch_prob[i][j] = 0.25 - (0.25*exp(-(4.0/3.0)*time*r));
				}
			}
		}
#	if defined (SHOW_TI_PROBS)
	printf ("Change probabilities (JC 69, time = %lf):\n", time);
	for (i=0; i<4; i++)
		{
		for (j=0; j<4; j++)
			{
			printf ("%lf ", ch_prob[i][j]);
			}
		printf ("\n");
		}
#	endif
}


int **IntMatrix(long nrl, long nrh, long ncl, long nch)
{
	long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
	int **d;

	/* allocate pointers to rows */
	d = (int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
	if (!d)
		{
		printf("Could not allocate pointer to rows for dmatrix!\n");
		exit(1);
		}
	d += NR_END;
	d -= nrl;
	
	/* allocate rows and set pointers to them */
	d[nrl] = (int *)malloc((size_t)(((nrow*ncol)+NR_END)*sizeof(int)));
	if (!d[nrl])
		{
		printf("Could not allocate rows and set pointers to them!\n");
		exit(1);
		}
	d[nrl] += NR_END;
	d[nrl] -= ncl;
	
	for(i = nrl + 1; i <= nrh; i++)
		{
		d[i] = d[i-1] + ncol;
		}
	
	return (d);
}


void Kimura2PChangeMat(double **ch_prob, double time, double k, double r)
{
	int i, j;
	double mu, X, v;
	
	mu = (4.0 / (2.0 + k));
	X = ((k+1.0)/2.0);
	v = time * r;

	for (i = 0; i < 4; i++)
		{
		for (j = 0; j < 4; j++)
			{
			if (i == j)																							/* i = j */
				{
				ch_prob[i][j] = 0.25 + (0.25 * exp(-(mu*v))) + (0.5 * exp(-(mu*v*X)));
				}
			else if ( (i == 0 && j == 2) || (i == 1 && j == 3) || (i == 2 && j == 0) || (i == 3 && j == 1) )	/* i != j, transition */
				{
				ch_prob[i][j] = 0.25 + (0.25 * exp(-(mu*v))) - (0.5 * exp(-(mu*v*X)));
				}
			else																								/* i != j, transversion */
				{
				ch_prob[i][j] = 0.25 - (0.25 * exp(-(mu*v)));
				}
			}
		}
#	if defined (SHOW_TI_PROBS)
	printf ("Change probabilities (K2P, time = %lf):\n", time);
	for (i=0; i<4; i++)
		{
		for (j=0; j<4; j++)
			{
			printf ("%lf ", ch_prob[i][j]);
			}
		printf ("\n");
		}
#	endif
}


void LabelTips (TreeNode *p)
{	
	if (p != NULL)
		{
		LabelTips (p->left);
		LabelTips (p->right);
		if (p->left == NULL && p->right == NULL)
			p->label = num++;
		}
}


void ParseArray(char *array)
{
	int i, j, first = YES;

	i = 0;
	do
		{
		if ( (array[i] == '' || array[i] == CRETURN || array[i] == NEWLINE || array[i] == HTAB || array[i] == VTAB || array[i] == SPACE || array[i] == SEMICOLON ||  array[i] == EQUALSIGN || array[i] == FORMFEED) && (first == YES))
			{
			array[i] = '|';
			first = NO;
			}
		else if ( (array[i] == '' || array[i] == CRETURN || array[i] == NEWLINE || array[i] == HTAB || array[i] == VTAB || array[i] == SPACE || array[i] == SEMICOLON ||  array[i] == EQUALSIGN || array[i] == FORMFEED) && (first == NO))
			{
			j = i;
			do
				{
				array[j] = array[j+1];
				j++;
				}while (array[j] != NULLTERM);
			i--;
			}
		else
			{
			first = YES;
			}
		i++;
		}while (array[i] != NULLTERM);

}


void PrintError(int error)
{
	char operation[1];

	if (error == 0)
		{
		printf("  ERROR: You must enter a string of integers.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 1)
		{
		printf("  ERROR: Undefined filename for the posterior predictive distribution\a\n");
		printf("         of parameters.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 2)
		{
		printf("  ERROR: Undefined filename for the posterior predictive distribution\a\n");
		printf("         of trees.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 3)
		{
		printf("  ERROR: Command not recognized.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 4)
		{
		printf("  ERROR: Model option unavailable. Valid options range between 0-3.\a\n");
		printf("  ACTION TAKEN: Model reset to the default (JC69).\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 5)
		{
		printf("  ERROR: Invalid option. Valid options are 'y' or 'n'.\a\n");
		printf("  ACTION TAKEN: Use gamma set to NO.\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 6)
		{
		printf("  ERROR: You have chosen to make ZERO draws. This is not valid.\a\n");
		printf("  ACTION TAKEN: Number of draws reset to default value (1000).\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 7)
		{
		printf("  ERROR: Number of draws must be a positive integer greater than 0.\a\n");
		printf("  ACTION TAKEN: Number of draws reset to default value (100).\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 8)
		{
		printf("  **WARNING** **WARNING** **WARNING** **WARNING** **WARNING** **WARNING**\a\n\n");
		printf("  Unless you have removed the burnin samples from BOTH the posterior trees\n");
		printf("  and parameters files choosing a burnin value of ZERO could produce invalid\n");
		printf("  inferences. The method is only valid when making draws from the output of\n");
		printf("  a chain at stationarity.\n\n");
		printf("  **WARNING** **WARNING** **WARNING** **WARNING** **WARNING** **WARNING**\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 9)
		{
		printf("  ERROR: Burnin must be ZERO or a positive value.\a\n");
		printf("  ACTION TAKEN: Burnin reset to default value (10000).\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 10)
		{
		printf("  ERROR: Seed should be a positive value.\a\n");
		printf("  ACTION TAKEN: Seed reset randomly.\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 11)
		{
		printf("  ERROR: Undefined output filename.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
	else if (error == 12)
		{
		printf("  ERROR: Undefined sequence data filename.\a\n");
		printf("  Type C and press RETURN to continue: ");	
		scanf("%s", &operation);
		}
}

void PrintHelp(int help)
{
	char operation[1];

	#if defined (WARRIOR_MAC)
		clrscr();
	#endif
	if (help == 0)								 /* Input files */
		{
		printf("\nINPUT FILE HELP:\n\n");
		printf("DATA FILE:\n\n");
		printf("\t\tThe input data file should be in a NEXUS or MrBayes (.mcmc) format. The\n");
		printf("\tfile must have a data block with commands defining the number of taxa and\n");
		printf("\tnumber of sites. This is necessary to determine the size of the data matrix.\n");
		printf("\tOnly the following DNA states are permitted; A, C, G, T. Gaps (-), ambiguous\n");
		printf("\tstates (N), unknown states (?), and states denoting a purine/pyrimadine (e.g.,\n");
		printf("\tR and Y) are not supported.\n\n\n");
		printf("POSTERIOR PARAMETERS FILE:\n\n");
		printf("\t\tThe program was written to use the Bayesian posterior approximation from\n");
		printf("\tMrBayes (Version 2.x or earlier; Huelsenbeck and Ronquist, 2001). MrBayes\n");
		printf("\tgenerates two different parameter files (.p and .bp). The .bp file and the\n");
		printf("\ttrees file are used by MAPPS. These, excluding the burnin, should represent\n");
		printf("\ta valid approximation of the posterior distribution. Be forewarned that the\n");
		printf("\tresults of MAPPS and any MCMC Bayesian analysis are dependent on the chain\n");
		printf("\tsampling from the target distribution (i.e., the chain has reached\n");
		printf("\tstationarity).\n\n\n");
		printf("BURNIN:\n\n");
		printf("\t\tThe burnin length is chosen by the user in the main menu and does not\n");
		printf("\tneed to removed manually. The program will exclude these samples from the\n");
		printf("\tanalysis.\n\n\n\n\n\n\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 1)							/* Output file */
		{
		printf("\nOUTPUT FILE HELP:\n\n");
		printf("\t\tA single results file is generated by MAPPS. This file contains a line\n");
		printf("\tfor each draw requested (i.e., number of replicates). The first line of the\n");
		printf("\tfile are the column headings describing the column's contents. Briefly, each\n");
		printf("\tline contains the following information (left to right), generation sampled,\n");
		printf("\tsubstitution model parameters, gamma shape parameter alpha, realized test\n");
		printf("\tstatistic for the observed data set, and the value of the test statistic for\n");
		printf("\tsimulated data sets. A comparison of posterior parameter estimates from the\n");
		printf("\tMrBayes parameter file (*.bp) for a sampled generation should be identical.\n\n");
		printf("\t\tUsers may wish to plot a histogram of the parameter values sampled by\n");
		printf("\tMAPPS and that of the original MrBayes posterior approximation. These distr-\n");
		printf("\tibutions should look identical. If the distributions are not nearly identical\n");
		printf("\tyou may wish to increase your sampling (number of draws) to insure that you\n");
		printf("\tare obtaining an unbiased sample of the posterior. Poor sampling of the\n");
		printf("\tposterior output of MrBayes OR unbiased sampling of a poor estimate of the\n");
		printf("\tposterior distribution may lead to VERY poor estimates of a model's post-\n");
		printf("\terior predictive p-value. This ultimately leads to model misspecification.\n");
		printf("\tThe sensitivity of the method to poor sampling or a poor estimate of the\n");
		printf("\tposterior distribution have not been studied yet.\n");
		printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 2)							/* Author contact information */
		{
		printf("\nAUTHOR'S CONTACT INFORMATION:\n\n");
		printf("\t\tJonathan P. Bollback\n");
		printf("\t\tDepartment of Biology\n");
		printf("\t\tUniversity of Rochester\n");
		printf("\t\tRochester, New York 14627 USA\n\n");
		printf("\t\tbollback@biomail.ucsd.edu\n");
		printf("\t\tbollback@brahms.biology.rochester.edu\n\n");
		printf("\tPlease send all comments, suggestions, questions, or bug reports to\n");
		printf("\tthe email addresses listed above or by the postal service (snail mail).\n");
		printf("\tI will do my very best to respond promptly to your emails.\n");
		printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 3)
		{
		printf("\nLIABILITY DISCLAIMER & SOURCE CODE/PROGRAM DISTRIBUTION:\n\n");
		printf("\t\tThe source code is provided as is with no waranty or gaurantee. The\n");
		printf("\tauthor (Jonathan Bollback) and any organization with which he is affiliated\n");
		printf("\tare not responsible in any way for the performance of the code, quality or\n");
		printf("\tvalidity of the results produced, or any anomaolous effects it may have on the\n");
		printf("\tsoftware users computer or information stored therein. This source code can be\n");
		printf("\tdistributed as is as long as this comment section is included and it is\n");
		printf("\tfor non-profit use. This code can not be sold or packaged on a media that is\n");
		printf("\tsold with out the permission of the author. This code can be freely modfied as\n");
		printf("\tlong as the following conditions are met: 1. The code is clearly commented\n");
		printf("\tindicating that it is a modification of MAPPS v1.x by an author other than me,\n");
		printf("\t2. The name of the modified software must clearly distinguish it from the\n");
		printf("\tunmodified version of MAPPS, 3. Modified versions can not be sold for profit,\n");
		printf("\t4. This disclaimer must be included in the modified code or executable clearly\n");
		printf("\taccessible to software users, and 5. Modified source code is freely available.\n");
		printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 4)
		{
		printf("\nACKNOWLEDGEMENTS:\n\n");
		printf("\tThe following individuals have given critical comments on the method\n");
		printf("and help with debugging the program code:\n\n");
		printf("\t\tJohn Huelsenbeck\n");
		printf("\t\tRasmus Nielsen\n");
		printf("\t\tFrederick Ronquist\n");
		printf("\t\tBret Larget\n");
		printf("\t\tKenneth Karol\n");
		printf("\t\tKelly Dyer\n");
		printf("\t\tAndrea Betancourt\n");
		printf("\t\tDaven Presgraves\n\n\n\n\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 5)
		{
		printf("\nHELP ANDREA:\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
	else if (help == 6)
		{
		printf("\nOVERVIEW OF METHOD:\n\n");
		printf("\tMAPPS is a program that implements a Bayesian model adequacy method\n");
		printf("using posterior predictive distributions. Specifically, the approach is that of\n");
		printf("posterior predictive checking by simulation. This approach is a Bayesian\n");
		printf("analog of such classical frequentist methods as the parametric bootstrap and\n");
		printf("randomization tests (see Rubin 1984). Briefly, the rational motivating this\n");
		printf("approach is that an adequate model should perform well in predicting future\n");
		printf("observations of the data or those observations of the data to be analyzed.\n");
		printf("The ability of a model to predict the data is evaluated by simulation using\n");
		printf("model parameter values, branch lengths, and topologies sampled from their\n");
		printf("respective marginal posterior distributions. If a model captures the underlying\n");
		printf("evolutionary process that generated the data then simulated data sets should\n");
		printf("on average look similar to the original data. Similarity between simulated and\n");
		printf("observed data sets is evaluated using the multinomial test statistic. This\n");
		printf("statistic quantifies the frequency of site patterns. Deviations between the\n");
		printf("true observations and simulated observations are an indicator of model misfit.\n");
		printf("A posterior predictive p-value is determined by repeated sampling of the\n");
		printf("posterior to generate a collection of simulated data sets. A comparison of the\n");
		printf("realized test statistic (original data) to the distribution of the test\n");
		printf("statistic for simulated data sets is a numerical approximation of the model's\n");
		printf("posterior predictive probability.\n");
		printf("\tFor a general review of the method see the citations listed below (Gelman et\n");
		printf("al. 1995; Gamerman 1997). For a more detailed treatment of the approach as it\n");
		printf("applies to the phylogeny model problem see Bollback (2002).\n\n\n");
		printf("REFERENCES:\n\n");
		printf("1. Gamerman, D. 1997. Markov Chain Monte Carlo: stochastic simulation for\n");
		printf("\tBayesian inference. Chapman and Hall, New York, USA.\n\n");
		printf("2. Gelman et al. 1995. Bayesian data analysis. Chapman and Hall, New York, USA.\n\n");
		printf("3. Rubin, D. 1984. Bayesianly justifiable and relevant frequency calculations for\n");
		printf("\tthe applied statistician. Ann. Stat. 12: 1151-1172.\n\n");
		printf("4. Bollback, J. 2002. Bayesian model adequacy and choice in phylogenetics.\n");
		printf("\tMol. Biol. Evol. 19(7): 1171-1180.\n\n\n\n\n\n");
		printf("Type C and press RETURN to go back to HELP MENU: ");	
		scanf("%s", &operation);
		}
}


void ReadFile (FILE *inOR, char *string2)		/* This function reads the data matrix out of a NEXUS file (MrBayes) - compatible with both UNIX and Mac files */
{
	int  j, offSet = 0;
	char *string, *tempString;

	string = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!string)
		{
		printf("ERROR: Problem allocating string in function 'ReadOriginalData'.\n");
		exit(1);
		}
	tempString = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!tempString)
		{
		printf("ERROR: Problem allocating string in function 'ReadOriginalData'.\n");
		exit(1);
		}
	tempString[0] = NULLTERM;

	/* Get string from file - placing contents into string2 */	
	while (fgets (string, MAXELEMS, inOR) != NULL)
		{
		offSet = strlen(string2);
		j = 0;
		do
			{
			string2[offSet++] = string[j++];
			}while(string[j] != NULLTERM);
		}
	string2[offSet+1] = NULLTERM;

	/* Parse string; all white spaces, tabs, semicolons, and returns are removed
	   leaving tokens seperated by '|' */

	ParseArray(string2);

	free(string);
	free(tempString);
}


int ReadParameterFile (double *parameters, int JC69, int HKY85, int K2P, int GTR, int *bigGen, int *sampleFreq, int getBiggest, int sample, int initial, int usegamma)
{
	int    i, j, k, l, n, m, numCommas, current_gen, error_skip=1, add_rate_cats=0;
	char   temp[10];

	if (getBiggest == YES)
		(*sampleFreq) = (*bigGen) = 0;

	i = 0;
	numCommas = 0;

	if (initial == YES)
		{
		if (usegamma == YES)	/* In the case where gamma is implememnted there should be an additional comma delimiting MrBayes estimate of alpha */
			add_rate_cats = 1;
		else
			add_rate_cats = 0;
		/* Check that model chosen is appropriate to parameter file */
		for (j = 0; j < MAXELEMS-4; j++)
			{
			if (paramFile[j] == 'd' && paramFile[j+1] == 'a' && paramFile[j+2] == 't' && paramFile[j+3] == 'a')
				{
				l = j+3;
				while (paramFile[l] != SEMICOLON)
					{
					if (paramFile[l] == COMMA)
						numCommas++;
					l++;
					}
				j = MAXELEMS-4;
				}
			}

		if (JC69 == 1 && numCommas != (6+add_rate_cats))
			{
			if (usegamma == YES)
				printf("ERROR: Parameter file does not appear to have the output of the JC69 + Gamma model.\a\n");	
			else
				printf("ERROR: Parameter file does not appear to have the output of the JC69 model.\a\n");	
			printf("  NOTE : This version of MAPPS has been only tested for MrBayes v1.x and v2.x.\n");
			error_skip = 0;
			}
		else if (K2P == 1 && numCommas != (7+add_rate_cats))	/* NOTE: This model could potentially be confounded with the HKY85 model, and vice versa. */
			{
			if (usegamma == YES)
				printf("ERROR: Parameter file does not appear to have the output of the K2P + Gamma model.\a\n");	
			else
				printf("ERROR: Parameter file does not appear to have the output of the K2P model.\a\n");	
			printf("  NOTE : This version of MAPPS has been only tested for MrBayes v1.x and v2.x.\n");
			error_skip = 0;
			}
		else if (HKY85 == 1 && numCommas != (7+add_rate_cats))
			{
			if (usegamma == YES)
				printf("ERROR: Parameter file does not appear to have the output of the HKY85 + Gamma model.\a\n");	
			else
				printf("ERROR: Parameter file does not appear to have the output of the HKY85 model.\a\n");	
			printf("  NOTE : This version of MAPPS has been only tested for MrBayes v1.x and v2.x.\n");
			error_skip = 0;
			}
		else if (GTR == 1 && numCommas != (12+add_rate_cats))
			{
			if (usegamma == YES)
				printf("ERROR: Parameter file does not appear to have the output of the GTR + Gamma model.\a\n");	
			else
				printf("ERROR: Parameter file does not appear to have the output of the GTR model.\a\n");	
			printf("  NOTE : This version of MAPPS has been only tested for MrBayes v1.x and v2.x.\n");
			error_skip = 0;
			}
		}

	numCommas = 0;
	if (error_skip == 1)
		{
		/* Get largest generations sampled value */
		if (getBiggest == YES)
			{
			for (j = 0; j < MAXELEMS-4; j++)
				{
				if (paramFile[j] == 'd' && paramFile[j+1] == 'a' && paramFile[j+2] == 't' && paramFile[j+3] == 'a')
					{
					k = (j+4);
					n = 0;
					while (paramFile[k] != COMMA)
						{
						temp[n] = paramFile[k];
						n++;
						k++;
						}
					temp[n] = '\0';
					(*sampleFreq) = (*bigGen);
					(*bigGen) = atoi(temp);
					}
				else if (paramFile[j] == '\0')
					break;
				}
			}
		else if (getBiggest == NO)
			{
			current_gen = 0;
			for (j = 0; j < MAXELEMS-4; j++)
				{
				if (paramFile[j] == 'd' && paramFile[j+1] == 'a' && paramFile[j+2] == 't' && paramFile[j+3] == 'a')
					{
					k = (j+4);
					n = 0;
					while (paramFile[k] != COMMA)
						{
						temp[n] = paramFile[k];
						n++;
						k++;
						}
					temp[n] = '\0';
					current_gen = atoi(temp);
					if (current_gen == sample)			/* Found correct line -- get parameters */
						{
						for (k = k; k < MAXELEMS-4; k++)
							{
							if (paramFile[k] == COMMA && numCommas >= 2)
								{
								if (JC69 == YES && usegamma == YES && numCommas < 6)
									{
									/* Skip to last comma - get alpha */
									do
										{
										k++;
										if (paramFile[k] == COMMA)
											numCommas++;
										}while (numCommas < 6);
									k++;
									m = 0;
									while (paramFile[k] != SEMICOLON)
										{
										if (paramFile[k] == PIPE)
											{
											k++;
											}
										else
											{
											temp[m++] = paramFile[k++];
											}
										}
									temp[m] = '\0';
									parameters[4] = atof(temp);
									numCommas++;
									}
								else if (HKY85 == YES)
									{
									/* Get kappa and basfreqs - start after three commas */
									m = 0;
									if (numCommas == 2)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[0] = atof(temp);
										}
									else if (numCommas == 3)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[1] = atof(temp);
										}
									else if (numCommas == 4)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[2] = atof(temp);
										}
									else if (numCommas == 5)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[3] = atof(temp);
										}
									else if (numCommas == 6)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[4] = atof(temp);
										}
									else if (numCommas == 7 && usegamma == YES)
										{
										k++;
										while (paramFile[k] != SEMICOLON)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[5] = atof(temp);
										}
									numCommas++;
									}
								else if (K2P == YES)
									{
									/* Get kappa after three commas */
									m = 0;
									if (numCommas == 2)
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[0] = atof(temp);
										}
									else if (numCommas == 8 && usegamma == YES)
										{
										k++;
										while (paramFile[k] != SEMICOLON)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										parameters[5] = atof(temp);
										}
									numCommas++;
									}
								else if (GTR == YES)
									{
									/* Get basefreqs and rate parameters of q-matrix */
									m = 0;
									if (numCommas == 2)						/* G => T */					
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("\n\nG=>T: %s\n", temp);
										parameters[0] = atof(temp);
										}
									else if (numCommas == 3)				/* C => T */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("C=>T: %s\n", temp);
										parameters[1] = atof(temp);
										}
									else if (numCommas == 4)				/* C => G */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("C=>G: %s\n", temp);
										parameters[2] = atof(temp);
										}
									else if (numCommas == 5)				/* A => T */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("A=>T: %s\n", temp);
										parameters[3] = atof(temp);
										}
									else if (numCommas == 6)				/* A => G */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("A=>G: %s\n", temp);
										parameters[4] = atof(temp);
										}
									else if (numCommas == 7)				/* A => C */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("A=>C: %s\n", temp);
										parameters[5] = atof(temp);
										}
									else if (numCommas == 8)				/* A */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("piA : %s\n", temp);
										parameters[6] = atof(temp);
										}
									else if (numCommas == 9)				/* C */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("piC : %s\n", temp);
										parameters[7] = atof(temp);
										}
									else if (numCommas == 10)				/* G */
										{
										k++;
										while (paramFile[k] != COMMA)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("piG : %s\n", temp);
										parameters[8] = atof(temp);
										}
									else if (numCommas == 11)				/* T */
										{
										k++;
										while (paramFile[k] != COMMA && paramFile[k] != SEMICOLON)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("piT : %s\n", temp);
										parameters[9] = atof(temp);
										if (usegamma == NO)
											break;
										}
									else if (numCommas == 12 && usegamma == YES)				/* Alpha */
										{
										k++;
										while (paramFile[k] != COMMA && paramFile[k] != SEMICOLON)
											{
											temp[m] = paramFile[k];
											k++;
											m++;
											}
										k--;
										temp[m] = '\0';
										//printf("Alpha : %s\n", temp);
										parameters[10] = atof(temp);
										break;
										}
									numCommas++;
									}
								}
							else if (paramFile[k] == COMMA)
								{
								numCommas++;
								}
							}
						break;
						}
					}
				else if (paramFile[j] == '\0')
					break;
				}	
			}
		}
	return(error_skip);
}


void ReadTreeFile (TreeNode *p, int nTaxa, int gen)
{
	int i, j, current_gen;
	char temp[20];

	current_gen = 0;
	for (j = 0; j < MAXELEMS-4; j++)
		{
		if (treeFile[j] == 't' && treeFile[j+1] == 'r' && treeFile[j+2] == 'e' && treeFile[j+3] == 'e')
			{
			do { j++; }while (treeFile[j] != PERIOD);
			i = 0;
			j++;
			while (treeFile[j] != SPACE)
				temp[i++] = treeFile[j++];
			temp[i] = NULLTERM;
			current_gen = atoi(temp);
			if (current_gen == gen)
				{
				do { j++; }while (treeFile[j] != LEFTPAR);
				i = 0;
				while (treeFile[j] != SEMICOLON)
					trees[i++] = treeFile[j++];
				trees[i++] = SEMICOLON;
				trees[i]   =  NULLTERM;
				j = (MAXELEMS-4);
				}
			}
		else if (treeFile[j] == NULLTERM)
			break;
		}

	SetUpTree(p, nTaxa);
	nodeIndex = 0;
	ReIndexNodes(root);
	//ShowTree(p, nTaxa);
}


void ReIndexNodes(TreeNode *p)
{
	if (p != NULL)
		{
		p->index = nodeIndex++;
		ReIndexNodes(p->left);
		ReIndexNodes(p->right);
		}
}


/*-------| RandomNumber |------------------------------------------------
|   This pseudorandom number generator is described in:
|   Park, S. K. and K. W. Miller.  1988.  Random number generators: good
|   ones are hard to find.  Communications of the ACM, 31(10):1192-1201.
*/
double Rnum(long int *seed)
{
	long int	lo, hi, test;

	hi = (*seed) / 127773;
	lo = (*seed) % 127773;
	test = 16807 * lo - 2836 * hi;
	if (test > 0)
		*seed = test;
	else
		*seed = test + 2147483647;
	return (double)(*seed) / (double)2147483647;
}


void SetUpTree(TreeNode *p, int nTaxa)
{
	int    i, k, counter, convert = NO, numNodes;
	char   temp_label[32], temp_len[32];
	TreeNode               *l, *r, *a;

	numNodes = (2 * nTaxa) - 1;
	
	/* Initialize structure to NULL */
	for (i = 0; i < numNodes; i++)
		{
		node[i].left		= NULL;
		node[i].right		= NULL;
		node[i].anc			= NULL;
		node[i].length		=  0.0;
		node[i].treeSum		=  0.0;
		node[i].index		=    i;
		node[i].label		=    0;
		node[i].flag		= -100;
		}

	i       = 0;
	counter = 0;
	while (trees[i] != SEMICOLON)								/* BUILD TREE FROM FILE */
		{
		if (trees[i] == LEFTPAR)									/* CREATE NODE - LEFT IF NULL ELSE RIGHT OR ROOT IF FIRST */
			{
			if (counter == 0)												/* root node */
				{
				p 	     = node + counter;
				root     = 				p;
				counter++;
				}
			else if (p->left == NULL && p->right == NULL)					/* left node */
				{
				l 		 = node + counter;
				p->left  = 				l;
				l->anc   =				p;
				p        =				l;
				counter++;
				}
			else if (p->left != NULL && p->right == NULL)					/* right node */
				{
				r 		 = node + counter;
				p->right = 				r;
				r->anc   =				p;
				p        =				r;
				counter++;
				}
			}
		else if (trees[i] == RIGHTPAR || trees[i] == COMMA)								/* MOVE DOWN TO THE ANC NODE */
			{
			p = p->anc;
			}
		else if (trees[i] == COLON)									/* READ IN BRANCHLENGTH AND ASSIGN TO CURRENT NODE */
			{
			for (k = (i+1); k < ((i+1)+32); k++)
				{
				if (WhatChar(trees[k]) == 0)
					temp_len[(k-(i+1))] = trees[k];
				else if (WhatChar(trees[k]) == 1)
					break;
				}
			temp_len[(k-(i+1))] = '\0';
			i = (k-1);
			if (p->anc == root && convert == YES)
				{
				p->length          = (atof(temp_len) / 2.0);
				root->left->length = (atof(temp_len) / 2.0);
				}
			else
				p->length = atof(temp_len);
			}
		else if ( CompareChar(trees[i]) == 1 )				/* READ TAXON LABEL - IN THIS CASE WILL BE INTEGER AND ADD NEW NODE */
			{
			for (k = i; k < (k+32); k++)
				{
				if (WhatChar(trees[k]) == 0)
					temp_label[k-i] = trees[k];
				else if (WhatChar(trees[k]) == 1)
					break;
				}
			temp_label[k-i] = '\0';
			i = (k-1);
			if (p->left == NULL && p->right == NULL)						/* left node */
				{
				l 		 = node + counter;
				p->left  = 				l;
				l->anc   =				p;
				p        =				l;
				counter++;
				}
			else if (p->left != NULL && p->right == NULL)					/* right node */
				{
				r 		 = node + counter;
				p->right = 				r;
				r->anc   =				p;
				p        =				r;
				counter++;
				}
			else if (p->left != NULL && p->right != NULL && p->anc == NULL)	/* at root: tree being input is unrooted -> action = convert to rooted */
				{
				r        = node + counter;									/* New node for tip taxa */
				counter++;
				a        = node + counter;									/* New root node for tree, conversion from unrooted to rooted */
				root     =              a;
				a->left  =              p;
				a->right =              r;
				p->anc   =              a;
				r->anc   =              a;
				p        =				r;
				convert  =			  YES;
				}
			p->label = atoi(temp_label);
			}
		else
			{
			/* Probably a white space - action = skip */
			}
		i++;
		}
	//printf("Tree structure set-up completed successfully...\n");
}


void SetQMatrix (int n, double **a, double *pi, double gtrRates[6])
{
	int			             i, j, k;
	double		    scaler, rates[6];
	
#	if defined (DEBUG_GTR)
	double sum;
#	endif	

	for (i = 0; i < 6; i++)
		rates[i] = 0.0;
		
	rates[0] = gtrRates[5];
	rates[1] = gtrRates[4];
	rates[2] = gtrRates[3];
	rates[3] = gtrRates[2];
	rates[4] = gtrRates[1];
	rates[5] = gtrRates[0];

	/* set Q matrix to 0 */
	for (i = 0; i < n; i++)
		{
		for (j = 0; j < n; j++)
			{
			a[i][j] = 0.0;
			}
		}

	k = 0;
	scaler = 0.0;
	for (i = 0; i < n; i++)
		{
		for (j = i+1; j < n; j++)
			{
			a[i][i] -= (a[i][j] = pi[j] * rates[k]);
			a[j][j] -= (a[j][i] = pi[i] * rates[k]);
			scaler  += (a[i][j] * pi[i]);
			scaler  += (a[j][i] * pi[j]);
			k++;
			}
		}

	/* Rescale Q matrix */
	scaler = 1.0 / scaler;
	for (i=0; i<n; i++)
		for (j=0; j<n; j++)
			a[i][j] *= scaler;
	
#	if defined (DEBUG_GTR)
	printf ("\nQ matrix:\n");
	for (i=0; i<n; i++)
		{
		sum = 0.0;
		for (j=0; j<n; j++)
			{
			sum += a[i][j];
			printf ("%lf ", a[i][j]);
			}
		printf("\t[%lf]\n", sum);
		}
#	endif
}


void ShowTree(TreeNode *p, int n)
{
	int i;

	printf("\nTree:");
	printf("\n\n");
	for (i = 0; i < (2*n) - 1; i++)
		{
		p = node + i;	
		if (p->left == NULL && p->right == NULL && p->anc != NULL) 									/*TIP*/
			printf ("%5d (NULL NULL %4d)\t[%2f] %d\n", p->index, p->anc->index, p->length, p->label);
		else if (p->left != NULL && p->right != NULL && p->anc != NULL) 							/*INTERNAL NODE*/
			printf ("%5d (%4d %4d %4d)\t[%2f]\n", p->index, p->left->index, p->right->index, p->anc->index, p->length);
		else																						/*ROOT*/
			printf ("%5d (%4d %4d NULL)\t[%2f]\n", p->index, p->left->index, p->right->index, p->length);
		}
	printf("\n\n");
}


void SimulateData(TreeNode *p, long int *seed, int nTaxa, int numSites, double bFreq[4], double alpha, double kappa, double gtrRates[6], double m, int JC69, int HKY85, int K2P, int GTR, int usegamma)
{
	int        n, c, i, j, k, nStates = 4, ancState, numNodes, which_to_use=0, *which_cat;
	double     rate=1.0, rn, sum, **q, **prob_A,**prob_B,**prob_C,**prob_D,gamma[NRATECATS], catFreq[NRATECATS];			/* Eventually gamma[] and catFreq[] needs to be dynamic allowing for a variety of rate categories */

	for (i = 0; i < NRATECATS; i++)
		gamma[i] = 1.0;

	upPassSeq  = (TreeNode **)malloc(sizeof(TreeNode *) * (2*nTaxa-1));
	if (!upPassSeq)
		{
		printf("ERROR: Memory allocation of upPassSeq failed!\n");
		exit(1);
		}
	which_cat  = (int*)malloc(sizeof(int) * numSites);
	if (!which_cat)
		{
		printf("ERROR: Memory allocation of which_cat failed!\n");
		exit(1);
		}

	k = 0;
	GetUpPassSequence(root, &k);

	numNodes = (2 * nTaxa) - 1;
	rate = 1.0;
	q    = psdmatrix(4);
	prob_A = psdmatrix(4);
	prob_B = psdmatrix(4);
	prob_C = psdmatrix(4);
	prob_D = psdmatrix(4);
	for (i = 0; i < STATES; i++)
		{
		for (j = 0; j < STATES; j++)
			{
			prob_A[i][j] = 0.0;
			prob_B[i][j] = 0.0;
			prob_C[i][j] = 0.0;
			prob_D[i][j] = 0.0;
			q[i][j] = 0.0;
			}
		}

	if (usegamma == YES)
		{
		DiscreteGamma (catFreq, gamma, alpha, alpha, NRATECATS, 0);
		#if defined (GAMMA_TWO)
		/* Choose a rate for each site in advance - this should insure that the rate is homogenous across the tree */
		for (c = 0; c < numSites; c++)
			{
			rn = Rnum(seed);
			sum = 0.0;
			for (i = 0; i < NRATECATS; i++)
				{
				sum += catFreq[i];
				if (rn < sum)
					{
					which_cat[c] = i;
					i = NRATECATS;
					}
				}
			}
		#endif
		}

	for (n = 0; n < numNodes; n++)
		{
		p = upPassSeq[n];
		if (GTR == YES)			/* Set Q matrix */
			SetQMatrix (4, q, bFreq, gtrRates);
		if (usegamma == NO && p->anc != NULL)
			{
			rate = 1.0;
			CalculateTransProb(q, prob_A, JC69, HKY85, K2P, GTR, (p->length)*m, rate, kappa, bFreq);
			}
		else if (usegamma == YES && p->anc != NULL)
			{
			rate = gamma[0];
			CalculateTransProb(q, prob_A, JC69, HKY85, K2P, GTR, (p->length)*m, rate, kappa, bFreq);
			rate = gamma[1];
			CalculateTransProb(q, prob_B, JC69, HKY85, K2P, GTR, (p->length)*m, rate, kappa, bFreq);
			rate = gamma[2];
			CalculateTransProb(q, prob_C, JC69, HKY85, K2P, GTR, (p->length)*m, rate, kappa, bFreq);
			rate = gamma[3];
			CalculateTransProb(q, prob_D, JC69, HKY85, K2P, GTR, (p->length)*m, rate, kappa, bFreq);
			}
		for (c = 0; c < numSites; c++)
			{
			if (p->anc == NULL)
				{
				rn = Rnum(seed);
				sum = 0.0;
				for (i = 0; i < STATES; i++)
					{
					sum += bFreq[i];
					if (rn < sum)
						{
						p->seq[c] = i;
						i = STATES;
						}
					}
				}
			else
				{
				if (usegamma == YES)
					{
					/* Choose a gamma rate category */
					#if defined (GAMMA_ONE)
					rn = Rnum(seed);
					sum = 0.0;
					for (i = 0; i < NRATECATS; i++)
						{
						sum += catFreq[i];
						if (rn < sum)
							{
							which_to_use = i;
							i = NRATECATS;
							}
						}
					#endif
					rn = Rnum(seed);
					sum = 0.0;
					ancState = p->anc->seq[c];
					for (i = 0; i < STATES; i++)
						{
						#if defined (GAMMA_ONE)
						if (which_to_use == 0)
							sum += prob_A[ancState][i];
						else if (which_to_use == 1)
							sum += prob_B[ancState][i];
						else if (which_to_use == 2)
							sum += prob_C[ancState][i];
						else if (which_to_use == 3)
							sum += prob_D[ancState][i];
						if (rn < sum)
							{
							p->seq[c] = i;
							i = STATES;
							}
						#endif
						
						#if defined (GAMMA_TWO)
						if (which_cat[c] == 0)
							sum += prob_A[ancState][i];
						else if (which_cat[c] == 1)
							sum += prob_B[ancState][i];
						else if (which_cat[c] == 2)
							sum += prob_C[ancState][i];
						else if (which_cat[c] == 3)
							sum += prob_D[ancState][i];
						if (rn < sum)
							{
							p->seq[c] = i;
							i = STATES;
							}
						#endif
						}
					}
				else if (usegamma == NO)
					{
					rn = Rnum(seed);
					sum = 0.0;
					ancState = p->anc->seq[c];
					for (i = 0; i < STATES; i++)
						{
						sum += prob_A[ancState][i];
						if (rn < sum)
							{
							p->seq[c] = i;
							i = STATES;
							}
						}
					}
				}
			}
		}
	free(upPassSeq);
	free(which_cat);
	free_psdmatrix(q);
	free_psdmatrix(prob_A);
	free_psdmatrix(prob_B);
	free_psdmatrix(prob_C);
	free_psdmatrix(prob_D);
}


void SimulatePosterior(TreeNode *p, long int *seed, int JC69, int HKY85, int K2P, int GTR, int burnin, int usegamma, int draws)
{
	int	      add_rate_cats=0, continue_or_return=0, numParams=0, i, j, k, n, nTaxa, nSites, num_greater, numReps, **originalMatrix, **simMatrix, bigGen, sampleFreq, numSamples_NoBurnin, gen, lines, position, num_lines_p, num_lines_t;
	double				    			     					             m, *parameters, bFreq[4], gtrRates[6], kappa=1.0, alpha=1.0, p_value;
	char 		yes='n', *string2;
	FILE 							         					 				                                            *inOR, *out;

	numReps =  	    draws;
	m       = TREE_HEIGHT;
	p_value = 0.0;
	num_greater = 0;

	printf("RUNNING MODEL ANALYIS...\n\n");
	printf("ANALYSIS PARAMETERS:\n\n");
	if (JC69 == YES)
		printf("\tModel                          : JC69\n");
	else if (HKY85 == YES)
		printf("\tModel                          : HKY85\n");
	else if (K2P == YES)
		printf("\tModel                          : K2P\n");
	else if (GTR == YES)
		printf("\tModel                          : GTR\n");
	if (usegamma == YES)
		yes = 'y';
	printf("\tGamma                          : %c\n", yes);
	printf("\tReplicates being performed     : %d\n", numReps);
	printf("\tGenerations to remove as burnin: %d\n", burnin);
	printf("\tOriginal data file             : %s\n", inOriginal);
	printf("\tParameter file                 : %s\n", inParamFile);
	printf("\tTrees file                     : %s\n\n", inTreeFile);
	printf("\tMAPPS results file             : %s\n\n", resultsName);
	fflush(stdout);

	if (usegamma == YES)
		add_rate_cats = 1;
	else
		add_rate_cats = 0;

	if (JC69 == YES)
		{
		parameters = (double *)malloc(sizeof(double) * (4 + add_rate_cats));
		numParams = 4 + add_rate_cats;
		}
	else if (HKY85 == YES)
		{
		parameters = (double *)malloc(sizeof(double) * (5 + add_rate_cats));
		numParams = 5 + add_rate_cats;
		}
	else if (K2P == YES)
		{
		parameters = (double *)malloc(sizeof(double) * (5 + add_rate_cats));
		numParams = 5 + add_rate_cats;
		}
	else if (GTR == YES)
		{
		parameters = (double *)malloc(sizeof(double) * (10 + + add_rate_cats));
		numParams = 10 + add_rate_cats;
		}

	if(!parameters)
		{
		printf("ERROR: Problem allocating parameter array.\n");
		exit(1);
		}
	if (JC69 == YES)
		{
		parameters[0] = 0.25;
		parameters[1] = 0.25;
		parameters[2] = 0.25;
		parameters[3] = 0.25;		
		}
	else if (K2P == YES)
		{		
		parameters[1] = 0.25;
		parameters[2] = 0.25;
		parameters[3] = 0.25;
		parameters[4] = 0.25;		
		}

	string2 = (char *)malloc(sizeof(char) * MAXELEMS);
	if (!string2)
		{
		printf("ERROR: Problem allocating string in function 'SimPosterior'.\n");
		exit(1);
		}
	string2[0] = NULLTERM;

	/* Open file stream */
	if ((inOR = fopen (inOriginal, "r+")) == NULL)     
		{
		printf ("ERROR: Could not open original data file.\n");
		exit (-1);
		}
	if ((out = fopen (resultsName, "a+")) == NULL)     
		{
		printf ("ERROR: Could not open results file.\n");
		exit (-1);
		}


	/* Get file information */
	ReadFile(inOR, string2);
	GetNumTaxaChars(&nTaxa, &nSites, string2);
	/* Allocate memory for matrices and other things */
	originalMatrix = IntMatrix(0, (long)nTaxa, 0, (long)nSites);
	simMatrix = IntMatrix(0, (long)nTaxa, 0, (long)nSites);
	node = (TreeNode *)malloc(sizeof(TreeNode) * 2 * nTaxa - 1);
	if (!node)
		{
		printf("ERROR: Memory allocation of nodes failed!\n");
		exit(1);
		}
	taxon_list = (ListNode *)malloc(sizeof(ListNode) * nTaxa);
	if (!taxon_list)
		{
		printf("ERROR: Memory allocation of taxon_list failed!\n");
		exit(1);
		}
	for (i = 0; i < (2 * nTaxa - 1); i++)
		{
		p = node + i;
		p->seq = (int *)malloc(sizeof(int) * nSites);
			if (!(p->seq))
				{
				printf("ERROR: Memory allocation of sequence array for node %d failed!\n", i);
				exit(1);
				}
		}
	/* Get more file information */
	GetLabelsMatrix(originalMatrix, nTaxa, nSites, string2);
	GetParameters(&num_lines_p);
	GetTrees(&num_lines_t);
	continue_or_return = ReadParameterFile(parameters, JC69, HKY85, K2P, GTR, &bigGen, &sampleFreq, YES, 0, YES, usegamma);

	if (continue_or_return == 1)
		{
		continue_or_return = ReadParameterFile(parameters, JC69, HKY85, K2P, GTR, &bigGen, &sampleFreq, YES, 0, NO, usegamma);

		S = nSites;
		T = nTaxa;

		#if defined (BASE_TEST)
			DetermineBaseFrequencies(nTaxa, nSites, originalMatrix, simMatrix, YES);
		#else
			DetermineSitePatterns(nTaxa, nSites, originalMatrix, simMatrix, YES);
		#endif

		/* Determine sampling frequency and offset for burnin */
		sampleFreq = bigGen - sampleFreq;
		if (burnin > bigGen)
			{
			printf("ERROR: Burnin (%d) can not be larger than number of samples from the posterior (%d).\n", burnin, bigGen);
			exit(1);
			}
		numSamples_NoBurnin = bigGen - burnin;					/* This value represents the number of samples at stationarity */
		lines = numSamples_NoBurnin / sampleFreq;				/* Number of lines to sample from - excluding burnin */

		printf("Replicate\t(Generation) -- \t(Realized T(X) Predictive T(X))\n");
		/* Sample from posterior distribution */
		for (i = 0; i < numReps; i++)
			{
			/* Draw a random number U[0, 1] such that you sample randomly from the posterior distribution at stationarity */
			gen = floor( Rnum(seed) * lines) + (burnin/sampleFreq);

			if ((i+1) % 100 == 0)
				printf("%9d\t(%10d) -- \t", i+1, gen*(sampleFreq));

			/* Read parameter file */
			continue_or_return = ReadParameterFile(parameters, JC69, HKY85, K2P, GTR, &bigGen, &sampleFreq, NO, gen*(sampleFreq), NO, usegamma);	

			/* Set basefreqs for JC69 and K2P */
			if (JC69 == YES || K2P == YES)
				{
				for (j = 0; j < 4; j++)
					bFreq[j] = 0.25;
				kappa = parameters[0];
				if (usegamma == YES && JC69 == YES)
					alpha = parameters[4];
				else if (usegamma == YES && K2P == YES)
					alpha = parameters[5];
				}
			else if (HKY85 == YES)
				{
				for (j = 1; j < 5; j++)
					bFreq[j-1] = parameters[j];
				kappa = parameters[0];
				if (usegamma == YES)
					alpha = parameters[5];
				}
			else if (GTR == YES)
				{
				for (j = 0; j < 6; j++)
					gtrRates[j] = parameters[j];
				for (j = 0; j < 4; j++)
					bFreq[j] = parameters[j+6];
				if (usegamma == YES)
					alpha = parameters[10];
				}

			/* Read tree file */
			ReadTreeFile(p, nTaxa, gen*(sampleFreq));
			/* Simulate data */
			SimulateData(root, seed, nTaxa, nSites, bFreq, alpha, kappa, gtrRates, m, JC69, HKY85, K2P, GTR, usegamma);
			downPassSeq  = (TreeNode **)malloc(sizeof(TreeNode *) * (2*nTaxa-1));
			if (!downPassSeq)
				{
				printf("ERROR: Memory allocation of downPassSeq failed!\n");
				exit(1);
				}
			
			k = 0;
			GetDownPassSequence(root, &k);
			for (k = 0; k < ((2*nTaxa)-1); k++)
				{
				p = downPassSeq[k];
				if (p->left == NULL && p->right == NULL)
					{
					position = (p->label - 1);
					for (n = 0; n < nSites; n++)
						simMatrix[position][n] = p->seq[n];
					}
				}
			free(downPassSeq);
			/* Calculate test statistic T(X|theta, tau) */
			#if defined (BASE_TEST)
				DetermineBaseFrequencies(nTaxa, nSites, originalMatrix, simMatrix, NO);
			#else
				DetermineSitePatterns(nTaxa, nSites, originalMatrix, simMatrix, NO);
			#endif
			if ((i+1) % 100 == 0)
				printf("(  %.3lf      %.3lf   )\n", sT, sS);
			if (sS > sT)
				num_greater++;
			
			fprintf(out, "%8d |\t", gen*(sampleFreq));
			for (k = 0; k < numParams; k++)
				{
				fprintf(out, "%.6lf\t", parameters[k]);
				}
			fprintf(out, "%.3lf\t%.3lf\n", sT, sS);
			}
			
		p_value = (double)num_greater/(double)numReps;	

		if (p_value > 0.950)
			p_value = 1.0 - p_value;
		
		if (JC69 == YES)
			{
			fprintf(out, "%s\tJC69\t%0.3lf\n", inOriginal, p_value);
			printf("\n\nAnalysis of JC69 model complete:\n\tPr(JC69) = %lf\n", p_value);
			}
		else if (K2P == YES)
			{
			fprintf(out, "%s\tK2P\t%0.3lf\n", inOriginal, p_value);
			printf("\n\nAnalysis of K2P model complete:\n\tPr(K2P) = %lf\n", p_value);
			}
		else if (HKY85 == YES)
			{
			fprintf(out, "%s\tHKY85\t%0.3lf\n", inOriginal, p_value);
			printf("\n\nAnalysis of HKY85 model complete:\n\tPr(HKY85) = %lf\n", p_value);
			}
		else if (GTR == YES)
			{
			fprintf(out, "%s\tGTR\t%0.3lf\n", inOriginal, p_value);
			printf("\n\nAnalysis of GTR model complete:\n\tPr(GTR) = %lf\n", p_value);
			}
		}
	Free_IntMatrix(originalMatrix, 0, 0);
	Free_IntMatrix(simMatrix, 0, 0);
	free(parameters);
	free(string2);
	free(taxon_list);
	free(node);
	fclose(inOR);
	fclose(out);
}


/* This function calculates S (unconstrained expectation under the multinomial) from Goldman (1993) 36:182-198. */
void TestStatistic(int nT, int nS, int nTaxa, int nSites, int **T, int **S, int *bT, int *bS, int original)
{
	int i;
	double X=0.0, num_chars=0.0;
	
	#if defined (BASE_TEST)
		num_chars = nTaxa * nSites;
		X =  (num_chars*log(num_chars));

		/* Calculate S */
		if (original == YES)
			{
			sT = 0.0;
			for (i = 0; i < 4; i++)
				{
				sT += (bT[i]*log(bT[i]));
				}
			sT = sT - X;
			}
		else if (original == NO)
			{
			sS = 0.0;
			for (i = 0; i < 4; i++)
				{
				sS += (bS[i]*log(bS[i]));
				}
			sS = sS - X;
			}
	#else
		X =  (nSites*log(nSites));

		/* Calculate S */
		if (original == YES)
			{
			sT = 0.0;
			for (i = 0; i < nT; i++)
				{
				sT += (T[nTaxa+1][i]*log(T[nTaxa+1][i]));
				}
			sT = sT - X;
			}
		else if (original == NO)
			{
			sS = 0.0;
			for (i = 0; i < nS; i++)
				{
				sS += (S[nTaxa+1][i]*log(S[nTaxa+1][i]));
				}
			sS = sS - X;
			}
		#endif
}


int WhatChar(char c)
{
	if (c == (char)COMMA)
		return (1);
	else if (c == (char)RIGHTPAR)
		return (1);
	else if (c == (char)LEFTPAR)
		return (1);
	else if (c == (char)COLON)
		return (1);
	else if (c == (char)SPACE)
		return (2);
	else
		return (0);
}


int WhichInt(char x)
{		
	if (x == 'A')
		return (0);
	else if (x == 'C')
		return (1);
	else if (x == 'G')
		return (2);
	else if (x == 'T')
		return (3);
	else
		{
		printf("ERROR: Problem converting character to integer state (%c).\n", x);		
		exit(1);
		}
	return(-999);
}


void WhichNuc(int x)
{		
	if (x == 0)
		printf("A");
	else if (x == 1)
		printf("C");
	else if (x == 2)
		printf("G");
	else if (x == 3)
		printf("T");
	else
		{
		printf("ERROR: Problem printing character state.\n");		
		exit(1);
		}
}